/***************************************************************************
 *  copyright (C) 2005                                                     *
 *  Umbrello UML Modeller Authors <uml-devel @uml.sf.net>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "autotqlayoutdlg.h"
#include "../associationwidget.h"
#include "../umlwidget.h"
#include "autotqlayout.h"
#include "newautotqlayoutdialog.h"
#include <tqcheckbox.h>
#include <tqspinbox.h>
#include <tqslider.h>
#include <kdebug.h>


AutotqlayoutDlg::AutotqlayoutDlg(KConfig* c,UMLView* v,TQWidget *tqparent, const char *name)
        :MyDialog1(tqparent, name)
{
    view=v;
    readConfig(c);
    config=c;
}

void AutotqlayoutDlg::slotSetAssociationWeight(int i)
{
    associationWeight=i;
}


void AutotqlayoutDlg::slotSetDependenciesWeight(int i)
{
    dependenciesWeight=i;
}


void AutotqlayoutDlg::slotSetGeneralizationWeight(int i)
{
    generalizationWeight=i;
}
void AutotqlayoutDlg::slotSetGenralizationAsEdges(bool b)
{
    genralizationAsEdges=b;
}


void AutotqlayoutDlg::slotSetDependenciesAsEdges(bool b)
{
    dependenciesAsEdges=b;
}


void AutotqlayoutDlg::slotSetAssociationAsEdges(bool b)
{
    associationAsEdges=b;
}



void AutotqlayoutDlg::slotSetCompressShapes(bool b)
{
    compressShapes=b;
}


void AutotqlayoutDlg::slotSetCenterDiagram(bool b)
{
    centerDiagram=b;
}


void AutotqlayoutDlg::slotSetClusterizeHierarchies(bool b)

{
    clusterizeHierarchies=b;
}


void AutotqlayoutDlg::slotSetShapeSeparation(int i)
{
    tqshapeSeparation=i;
}

void AutotqlayoutDlg::slotReloadSettings()
{
    readConfig(config);
}


void AutotqlayoutDlg::slotSaveSettings()
{
    writeConfig(config);
}


void AutotqlayoutDlg::slotDoAutotqlayout()
{

    Autotqlayout::Autotqlayouter* a=getAutotqlayouter();;

    a->setAssociationAsEdges( associationAsEdges);
    a->setAssociationWeight( associationWeight );
    a->setCenterDiagram( centerDiagram);
    a->setDependenciesAsEdges( dependenciesAsEdges);
    a->setClusterizeHierarchies( clusterizeHierarchies);
    a->setCompressShapes( compressShapes);
    a->setDependenciesWeight( dependenciesWeight);
    a->setGeneralizationAsEdges( genralizationAsEdges);
    a->setGeneralizationWeight( generalizationWeight);
    a->setNoteConnectionWeight( 1);
    a->setNoteConnectionsAsEdges(true);
    a->setShapeSeparation( tqshapeSeparation);
    a->autotqlayout( view);
    delete a;
    a=0;
    accept();
}

void AutotqlayoutDlg::readConfig( KConfig * conf)
{
    conf->setGroup("AutotqlayoutDlg");
    associationEdgesCB->setChecked((bool)(conf->readBoolEntry( "associationAsEdges",false)));
    centerDiagramCB->setChecked((bool)(conf->readBoolEntry( "centerDiagram",true)));
    dependenciesEdgesCB->setChecked((bool)(conf->readBoolEntry( "dependenciesAsEdges",false)));
    clusterizeHierarchiesCB->setChecked((bool)(conf->readBoolEntry( "clusterizeHierarchies",false)));
    compressShapesCB->setChecked((bool)(conf->readBoolEntry( "compressShapes",true)));
    dependenciedEdgesSL->setValue((int)(conf->readNumEntry( "dependenciesWeight",0)));
    generalizationCB->setChecked((bool)(conf->readBoolEntry( "genralizationAsEdges",true)));
    generalizationEdgessSL->setValue((int)(conf->readNumEntry( "generalizationWeight",1)));
    associationEdgesSL->setValue((int)(conf->readNumEntry( "associationWeight",0)));
    tqshapeSeparationSB->setValue((int)(conf->readNumEntry( "tqshapeSeparation",0)));
    algorithmCOB->setCurrentItem((int)(conf->readNumEntry( "algorithm",0)));
}

void AutotqlayoutDlg::writeConfig( KConfig * conf)
{
    //  conf=kapp->config();
    conf->setGroup("AutotqlayoutDlg");
    conf->writeEntry( "associationAsEdges",associationEdgesCB->isChecked());
    conf->writeEntry( "centerDiagram", centerDiagramCB->isChecked());
    conf->writeEntry("dependenciesAsEdges",dependenciesEdgesCB->isChecked());
    conf->writeEntry("clusterizeHierarchies",clusterizeHierarchiesCB->isChecked());
    conf->writeEntry("dependenciesWeight", dependenciedEdgesSL->value());
    conf->writeEntry("genralizationAsEdges",generalizationCB->isChecked());

    conf->writeEntry("generalizationWeight",generalizationEdgessSL->value());
    conf->writeEntry("associationWeight",associationEdgesSL->value());
    conf->writeEntry("tqshapeSeparation",tqshapeSeparationSB->value());
    //conf->writeEntry("al


    //algorithmCOB->setCurrentItem(conf->readNumEntry( "algorithm",0));

}

void AutotqlayoutDlg::slotSelectAlgorithm( const TQString& _algname)
{
    algname=_algname;

}

Autotqlayout::Autotqlayouter * AutotqlayoutDlg::getAutotqlayouter( )
{
    const TQString text = algorithmCOB->currentText();
    kDebug() << "Autotqlayout Algorithm " << algname << " found " << text << endl;
    if (text == "dot")
        return new Autotqlayout::DotAutotqlayouter();
    if (text == "circo")
        return new Autotqlayout::CircoAutotqlayouter();
    if (text ==  "neato")
        return new Autotqlayout::NeatoAutotqlayouter();
    kError() << "Autotqlayout Algorithm not found" << endl;
    return new Autotqlayout::DotAutotqlayouter();
}



#include "autotqlayoutdlg.moc"
